/* Defragmenting netfilter module.  Whee! */
#define EXPORT_SYMTAB
#include <linux/module.h>
#include <linux/netfilter.h>
#include <linux/netfilter_ipv4.h>
#include <linux/ip.h>
#include <net/ip.h>

static unsigned int ip_defrag_fn(unsigned int hooknum,
				 struct sk_buff **pskb,
				 const struct net_device *in,
				 const struct net_device *out,
				 int (*okfn)(struct sk_buff *))
{
	(*pskb)->nfcache |= NFC_IP_FRAG;

	if ((*pskb)->nh.iph->frag_off & htons(IP_MF|IP_OFFSET)) {
		*pskb = ip_defrag(*pskb);
		if (!*pskb)
			return NF_STOLEN;
		ip_send_check((*pskb)->nh.iph);
		(*pskb)->nfcache |= NFC_UNKNOWN | NFC_ALTERED;
	}

	return NF_ACCEPT;
}

/* We have to refragment because noone expects locally-generated
   packets to be over the mtu. */
static unsigned int ip_refrag_fn(unsigned int hooknum,
				 struct sk_buff **pskb,
				 const struct net_device *in,
				 const struct net_device *out,
				 int (*okfn)(struct sk_buff *))
{
	struct rtable *rt = (struct rtable *)(*pskb)->dst;

	if ((*pskb)->len > rt->u.dst.pmtu) {
		/* No hook can be after us, so this should be OK. */
		ip_fragment(*pskb, okfn);
		return NF_STOLEN;
	}
	return NF_ACCEPT;
}

/* Before filtering on pre-routing and local-out hooks. */
static struct nf_hook_ops ip_defrag_ops[3]
= { { { NULL, NULL }, ip_defrag_fn, NULL, PF_INET, NF_IP_PRE_ROUTING, -1 },
    { { NULL, NULL }, ip_defrag_fn, NULL, PF_INET, NF_IP_LOCAL_OUT, -1 },
    { { NULL, NULL }, ip_refrag_fn, NULL, PF_INET, NF_IP_POST_ROUTING, INT_MAX}
};


int init_module(void)
{
	int ret;

	ret = nf_register_hook(&ip_defrag_ops[0]);
	if (ret == 0) {
		ret = nf_register_hook(&ip_defrag_ops[1]);
		if (ret == 0) {
			ret = nf_register_hook(&ip_defrag_ops[2]);
			if (ret != 0) {
				nf_unregister_hook(&ip_defrag_ops[1]);
				nf_unregister_hook(&ip_defrag_ops[0]);
			}
		} else {
			nf_unregister_hook(&ip_defrag_ops[0]);
		}
	}

	return ret;
}

void cleanup_module(void)
{
	nf_unregister_hook(&ip_defrag_ops[0]);
	nf_unregister_hook(&ip_defrag_ops[1]);
	nf_unregister_hook(&ip_defrag_ops[2]);
}
